#INCLUDE "HMG.CH"
#INCLUDE "COMMON.CH"

STATIC xVersion := "v0.10"

MEMVAR Button_Open
MEMVAR cLastDir

*+--------------------------------------------------------------------
*+
*+    Procedure MAIN()
*+
*+--------------------------------------------------------------------
*+
PROCEDURE MAIN
PUBLIC Button_Open
PUBLIC cLastDir := GetCurrentFolder()

   SET CENTURY ON
   SET DATE GERMAN
   SET EPOCH TO YEAR( DATE() ) - 50

   IF !IsWindowDefined( MainForm )
      Load Window MainForm

      ON KEY ESCAPE OF MainForm ACTION MainForm.Release
      CENTER WINDOW MainForm

      Activate Window MainForm
   ENDIF

RETURN

*+--------------------------------------------------------------------
*+
*+    Procedure OpenDir()
*+
*+--------------------------------------------------------------------
*+
PROCEDURE OpenDir( cInitPath )

LOCAL cXls             := ""
LOCAL cTitle           := "open Excel XLS " + xVersion
LOCAL lNewFolderButton := .F.

   DEFAULT cInitPath TO cLastDir

   cXls := GetFile( { { "Excel Sheet", "*.xls*" } }, cTitle, TRIM( cInitPath ) )
   IF !EMPTY( cXls )
      cLastDir := SUBSTR( cXls, 1, RAT( "\", cXls ) )
      OpenXls( cXls )
   ENDIF

RETURN

*+--------------------------------------------------------------------
*+
*+    Static Procedure OpenXls()
*+
*+    Called from ( dbfexcel.prg )   1 - procedure opendir()
*+
*+--------------------------------------------------------------------
*+
STATIC PROCEDURE OpenXls( cXls )

LOCAL oExcel
LOCAL oWorkBook
LOCAL oSheet
LOCAL oRange
LOCAL oError
LOCAL bSaveError
LOCAL i, cType
LOCAL numRows
LOCAL numColumns
LOCAL cEnde
LOCAL Retvar     := .F.
LOCAL aExcel     := {}
LOCAL aWide      := {}
LOCAL aFormat    := {}
LOCAL aClone

   IF ( oExcel := win_oleCreateObject( "Excel.Application" ) ) == NIL
      MsgInfo( 'Error: Microsoft Excel is not installed, ', win_OleErrorText() )
      RETURN
   ENDIF

   MainForm.Button_Open.hide()

   // save Errorblock
   bSaveError := ERRORBLOCK()

   // set new Errorblock
   ERRORBLOCK( { | e | BREAK( e ) } )
   BEGIN SEQUENCE

      // remember using SET CENTURY ON / SET DATE xxx

      oExcel:Application:visible := .F.

      // disable warnings
      oExcel:Application:DisplayAlerts := .F.

      // open XLS-Table in Workbook
      oExcel:Application:workbooks:Open( cXls )

      // Make the first one active
      oWorkBook := oExcel:activeWorkBook
      oExcel:Application:Worksheets( 1 ) :activate()

      // Speed things up by creating an object containing the cells
      oSheet := oExcel:Worksheets( 1 ) :cells

      // count ROW and COL
      numRows := oWorkBook:workSheets( 1 ) :usedRange:Rows:Count
      numColumns := oWorkBook:workSheets( 1 ) :usedRange:Columns:Count

      // create Array with wanted Dimension
      i := 1
      FOR i := 1 TO numRows
         AADD( aExcel, ARRAY( numColumns ) )
      NEXT
      cEnde := ZAHL2CHR( numColumns )

      // fit all Column to optimal size
      //       oSheet:EntireColumn:AutoFit()
      oSheet:Columns( "A:" + cEnde ) :autoFit()

      // transfer to Array
      aExcel := oSheet:range( "A1:" + cEnde + LTRIM( STR( numRows ) ) ) :value

      // get o:ColumnWidth and o:NumberFormat
      FOR i := 1 TO numColumns
         AADD( aWide, oSheet:Columns( i ) :ColumnWidth )
         AADD( aFormat, oSheet:Columns( i ) :NumberFormat )
      NEXT

      // close Excel
      oExcel:Quit()

      // we need to ACLONE() Array while it is only a Reference
      aClone := ACLONE( aExcel )
      // to "close" EXCEL
      oExcel := NIL

      // now show Array in GRID
      Array2GRID( aClone, cXls, aWide, aFormat )

   RECOVER USING oError
      MsgInfo( "Error Excel ..." )
   END SEQUENCE
   // restore Errorblock
   ERRORBLOCK( bSaveError )

   MainForm.Button_Open.show()

RETURN

*+--------------------------------------------------------------------
*+
*+    Static Procedure Array2GRID()
*+
*+    Called from ( dbfexcel.prg )   1 - static procedure openxls()
*+
*+--------------------------------------------------------------------
*+
STATIC PROCEDURE Array2GRID( aExcel, cXls, aWide, aFormat )

LOCAL i, iMax
LOCAL j, jMax
LOCAL nEnde
LOCAL aHead      := {}
LOCAL aWidths    := {}
LOCAL aLine      := {}
LOCAL aNew       := {}
LOCAL nStart     := 1
LOCAL aBackColor := {}
LOCAL aForeColor := {}
LOCAL aClickHead := {}
LOCAL cDBF       := GetCurrentFolder() + "\Template.DBF"
LOCAL bBackColor
LOCAL bForeColor
LOCAL bClickHead
   // LOCAL aDir := DIRECTORY("*.*")      // use as Array Sample for GRID

   iMax := LEN( aExcel )
   FOR i := 1 TO iMax
      AADD( aHead, aExcel[ i ] [ 1 ] )
      AADD( aWidths, aWide[ i ] * 10 )    // twips ...
   NEXT

   iMax := LEN( aExcel )                  // Rows
   jMax := LEN( aExcel[ 1 ] )             // Columns
   j := 1
   FOR j := 1 TO jMax                     // every Column
      aLine := {}
      i := 1
      FOR i := 1 TO iMax                  // build Line

         IF NIL = aExcel[ i ] [ j ]       // if NIL -> ""
            aExcel[ i ] [ j ] := ""
         ENDIF

         AADD( aLine, aExcel[ i ] [ j ] ) // Line Array
      NEXT
      //  IF J <> 1                       // no Headerline
      AADD( aNew, ACLONE(aLine))
      //  ENDIF
   NEXT

   nEnde := LEN( aNew )

   bBackColor := { || IF( ( This.CellRowIndex < nStart .OR. This.CellRowIndex > nEnde ), ;
                   { 155, 0, 0 }, { 255, 255, 255 } ) }
   bForeColor := { || IF( ( This.CellRowIndex < nStart .OR. This.CellRowIndex > nEnde ), ;
                   { 255, 255, 255 }, { 0, 0, 0 } ) }

   bClickHead := { || SetEndCol( aNew, @iMax ) }

   jMax := LEN( aHead )
   j := 1
   FOR j := 1 TO jMax                     // every Column
      AADD( aBackColor, bBackColor )
      AADD( aForeColor, bForeColor )
      AADD( aClickHead, bClickHead )
   NEXT

   DEFINE WINDOW Form_1 ;
           AT 0, 0 ;
           WIDTH 1280 ;
           HEIGHT 1024 ;
           TITLE cXls ;
           ICON "d:\ALASKA\RES\OHR2.ICO" ;
           FONTNAME "Calibri" ;
           FONTSIZE 11 ;
           ON MAXIMIZE Form_1_Adjust( "Form_1" ) ;
           ON SIZE Form_1_Adjust( "Form_1" ) ;
           ON INIT InitForm1() ;
           ON RELEASE CloseForm1( cDBF )

   DEFINE TOOLBAR oTOOLBAR ;
           OF Form_1 ;
           RIGHTTEXT ;
           BUTTONSIZE 64, 32

   BUTTON oBtSave ;
           CAPTION "F4: Save 2 DBF" ;
           PICTURE "d:\ALASKA\RES\Save32.ico" ;
           NOTRANSPARENT ;
           ACTION Save2Dbf( aNew, aHead, aWidths, cXls, @nStart, @nEnde, @cDBF, aFormat, iMax )

   END TOOLBAR

   @ 50, 10 GRID Grid_1 ;
           OF Form_1 ;
           FONT "Calibri" ;
           SIZE 11 ;
           WIDTH 1240 ;
           HEIGHT 900 ;
           HEADERS aHead ;
           WIDTHS aWidths ;
           ITEMS aNew ;
           TOOLTIP "set End-Row with double-click" ;
           DYNAMICBACKCOLOR aBackColor ;
           DYNAMICFORECOLOR aForeColor ;
           ON DBLCLICK SetEndRow( aNew, @nEnde ) ;
           ON HEADCLICK aClickHead

   END WINDOW


   ON KEY F4 OF Form_1 ACTION Save2Dbf( aNew, aHead, aWidths, cXls, @nStart, @nEnde, @cDBF, aFormat, iMax )
   ON KEY ESCAPE OF Form_1 ACTION Form_1.Release

   Form_1.grid_1.setfocus
   CENTER WINDOW Form_1
   ACTIVATE WINDOW Form_1

RETURN

*+--------------------------------------------------------------------
*+
*+    Static Procedure InitForm1()
*+
*+    Called from ( dbfexcel.prg )   1 - static procedure array2grid()
*+
*+--------------------------------------------------------------------
*+
STATIC PROCEDURE InitForm1()

   // to get DYNAMICxxxxCOLOR work on 1t time
   //    Form_1.grid_1.hide()
   //    Form_1.grid_1.show()

   Form_1.grid_1.PaintDoubleBuffer := .T.
   Form_1.grid_1.EnableUpdate()
   Form_1.grid_1.Refresh()
   Form_1.grid_1.setfocus()
RETURN

*+--------------------------------------------------------------------
*+
*+    Static Procedure Form_1_Adjust()
*+
*+    Called from ( dbfexcel.prg )   2 - static procedure array2grid()
*+
*+--------------------------------------------------------------------
*+
STATIC PROCEDURE Form_1_Adjust( cName )

LOCAL nWidth
LOCAL nHeight

   nWidth := GetProperty( cName, "Width" )
   nHeight := GetProperty( cName, "Height" )

   Setproperty( cName, "Grid_1", "Width", nWidth - 40 )
   Setproperty( cName, "Grid_1", "Height", nHeight - 90 - 40 )

RETURN

*+--------------------------------------------------------------------
*+
*+    Procedure CloseForm1()
*+
*+    Called from ( dbfexcel.prg )   1 - static procedure array2grid()
*+
*+--------------------------------------------------------------------
*+
PROCEDURE CloseForm1( cDBF )

   RELEASE KEY F4 OF Form_1
   RELEASE KEY ESCAPE OF Form_1

   Form_1.Grid_1.DYNAMICBACKCOLOR := NIL
   Form_1.Grid_1.DYNAMICFORECOLOR := NIL

   IF FILE( cDBF )
      FERASE( cDBF )
   ENDIF

   MainForm.Button_Open.show()

RETURN

*+--------------------------------------------------------------------
*+
*+    Static Function ZAHL2CHR()
*+
*+    Called from ( dbfexcel.prg )   1 - static procedure openxls()
*+
*+--------------------------------------------------------------------
*+
STATIC FUNCTION ZAHL2CHR( numColumns )

LOCAL nMal
LOCAL cEnde

   IF numColumns > 26
      nMal := INT( numColumns / 26 )
      cEnde := CHR( nMal + 64 ) + CHR( ( numColumns - ( nMal * 26 ) ) + 64 )
   ELSE
      cEnde := CHR( numColumns + 64 )
   ENDIF

RETURN cEnde

*+--------------------------------------------------------------------
*+
*+    Static Procedure SetEndRow()
*+
*+    Called from ( dbfexcel.prg )   1 - static procedure array2grid()
*+
*+--------------------------------------------------------------------
*+
STATIC PROCEDURE SetEndRow( aNew, nInOut )

LOCAL nRow := This.CellRowIndex
LOCAL iMax := LEN( aNew )

   IF nRow <= iMax
      nInOut := nRow
      MsgInfo( "End Row : " + LTRIM( STR( nRow ) ) + " of " + LTRIM( STR( iMax ) ) )
   ENDIF
RETURN

*+--------------------------------------------------------------------
*+
*+    Static Procedure SetEndCol()
*+
*+    Called from ( dbfexcel.prg )   1 - static procedure array2grid()
*+
*+--------------------------------------------------------------------
*+
STATIC PROCEDURE SetEndCol( aNew, nInOut )

LOCAL nCol := This.CellColIndex           // BUG Col+1
LOCAL iMax := LEN( aNew[ 1 ] )

   IF nCol <= iMax
      nInOut := nCol
      MsgInfo( "End Col : " + ;
               LTRIM( STR( nCol ) ) + " of " + ;
               LTRIM( STR( iMax ) ) )
   ENDIF
RETURN

PROCEDURE Save2Dbf( aNew, aHead, aWidths, cXls, nStart, nEnde, cDBF, aFormat, iMax )
   MsgInfo("under construction")
RETURN
